type EventKeyWithNoArg<T> = {
  [K in keyof T]: T[K] extends void ? K : never
}[keyof T]

type CallbackSignature<T, K extends keyof T> = K extends EventKeyWithNoArg<T>
  ? () => void
  : (value: T[K]) => void

export class EventEmitter<T extends Record</* eventName */ string, /* eventArg */ any>> {
  #listeners = new Map<keyof T, Set<(...args: any[]) => any>>()

  #getList(key: keyof T) {
    if (!this.#listeners.has(key)) {
      this.#listeners.set(key, new Set())
    }
    return this.#listeners.get(key)!
  }

  on<K extends keyof T>(key: K, fn: CallbackSignature<T, K>) {
    this.#getList(key).add(fn)
    return this
  }

  off<K extends keyof T>(key: K, fn: CallbackSignature<T, K>) {
    this.#getList(key).delete(fn)
    return this
  }

  once<K extends keyof T>(key: K, fn: CallbackSignature<T, K>) {
    const wrapped = (...arg: any[]) => {
      ;(fn as any)(...arg)
      this.off(key, wrapped)
    }
    return this.on(key, wrapped)
  }

  emit<K extends EventKeyWithNoArg<T>>(key: K): this
  emit<K extends keyof T>(key: K, value: T[K]): this

  emit<K extends keyof T>(key: K, value?: T[K]) {
    const listeners = this.#getList(key)
    if (arguments.length === 1) {
      listeners.forEach(fn => fn())
    } else {
      listeners.forEach(fn => fn(value!))
    }
    return this
  }
}
