import { resolve, extname } from "path"
import invariant from "tiny-invariant"
import type { IFileSystem } from "./adapters"
import { once } from "./util"

export type OnePasswordFileManager = ReturnType<typeof OnePasswordFileManager>

export function OnePasswordFileManager(
  fs: IFileSystem,
  path: string,
  profileName: string
) {
  const root = resolve(path, profileName)
  invariant(fs.existsSync(path), `Path ${path} does not exist.`)
  invariant(fs.existsSync(root), `Profile ${profileName} does not exist.`)

  const abs = (path: string) => resolve(root, path)

  const result = {
    getProfile() {
      return fs.readFile(abs("profile.js"))
    },

    getFolders() {
      return fs.readFile(abs("folders.js"))
    },

    async getAttachments() {
      const files = await fs.readdir(root)
      files
        .filter(name => extname(name) === ".attachment")
        .forEach(name => {
          const sep = name.indexOf("_")
          const path = resolve(root, name)
          const [itemUUID, fileUUID] = [name.slice(0, sep), name.slice(sep + 1)]
          return {
            itemUUID,
            fileUUID,
            getFile: once(() => fs.readFile(path)),
          }
        })
    },

    async getBand(name: string) {
      const path = abs(`band_${name}.js`)
      if (fs.existsSync(path)) {
        return await fs.readFile(path)
      }
    },

    async setProfile(profile: string) {
      await fs.writeFile("profile.js", profile)
    },

    async setFolders(folders: string) {
      await fs.writeFile("folders.js", folders)
    },

    async setBand(name: string, band: string) {
      await fs.writeFile(`band_${name}.js`, band)
    },
  }
  return result
}
